% This file should run after the main gendata* file. It will generate the 
% SHELFICEloadAnomalyFile = 'phi0surf.exp1.jmd95z',
% in data.shelfice

eos = 'jmd95z';

dz = delta_z*ones(1,nz);
zgp1 = [0,cumsum(dz)];
zc = .5*(zgp1(1:end-1)+zgp1(2:end));
zg = zgp1(1:end-1);
dz = diff(zgp1);
sprintf('delZ = %d * %7.6g,',nz,dz);

% Gravity
gravity=9.81;
rhoConst = 1030;
% compute potential field underneath ice shelf
talpha = 2e-4;
sbeta  = 7.4e-4;
tref = -1.80*ones(nz,1);
t    = tref;


% 20200802- use our own initial stratification profile
% linear stratification
generate_stratification;


sref = salt_range';
s    = sref;
gravity = 9.81;
k=1;
dzm = abs([zg(1)-zc(1) .5*diff(zc)]);
dzp = abs([.5*diff(zc) zc(end)-zg(end)]);
p = abs(zc)*gravity*rhoConst*1e-4;
dp = p;
kp = 0;
while rms(dp) > 1e-13
    phiHydF(k) = 0;
    p0 = p;
    kp = kp+1;
    for k = 1:nz
        switch eos
            case 'linear'
                drho = rhoConst*(1-talpha*(t(k)-tref(k))+sbeta*(s(k)-sref(k)))-rhoConst;
            case 'jmd95z'
                drho = densjmd95(s(k),t(k),p(k))-rhoConst;
            case 'mdjwf'
                drho = densmdjwf(s(k),t(k),p(k))-rhoConst;
            otherwise
                error(sprintf('unknown EOS: %s',eos))
        end
        phiHydC(k)   = phiHydF(k) + dzm(k)*gravity*drho/rhoConst;
        phiHydF(k+1) = phiHydC(k) + dzp(k)*gravity*drho/rhoConst;
    end
    switch eos
        case 'mdjwf'
            p = (gravity*rhoConst*abs(zc) + phiHydC*rhoConst)/gravity/rhoConst;
    end
    dp = p-p0;
end

% % % % % icetopo = ones(nx,1)*min(Hmax,Hmin + dHdx*(latc-latg(1)));
% % % % % fid=fopen('icetopo.exp1','w','b'); fwrite(fid,icetopo,acc);fclose(fid);
% % % % % fid=fopen('pload.exp1','w','b'); fwrite(fid,-icetopo,acc);fclose(fid);


% After modifying the code in calc_phi_hyd.F on Apr26,2012 this is the
% consistent way of computing phi0surf. For this, we need the grid
% information (hFacC's). For convenience, it's taken from a previous model
% run.
%
% The way of computing phi0surf consistent with code prior to Apr26,2012
% is recovered by setting drloc*dphi=0
g.HFacC=ncread('grid_glued.nc','HFacC');
msk=sum(g.HFacC,3); msk(msk>0)=1;
phi0surf = zeros(size(bathy_combined));
for ix=1:size(bathy_combined,1)
  for iy=1:size(bathy_combined,2)
    k=max(find(abs(zg)<abs(icetopo(ix,iy))));
    if isempty(k)
      k=0;
    end
    if k>0
      kp1=min(k+1,nz);
      drloc=1-g.HFacC(ix,iy,k);
      %drloc=(abs(icetopo(ix,iy))-abs(zg(k)))/dz(k);
      dphi = phiHydF(kp1)-phiHydF(k);
      phi0surf(ix,iy) = (phiHydF(k)+drloc*dphi)*rhoConst*msk(ix,iy);
    end
  end
end
fid=fopen(['phi0surf.exp1.' eos],'w','b'); fwrite(fid,phi0surf,acc);fclose(fid);