clear
close all
% This is a matlab script that generates the input data

% the configuation approximately the ISOMIP experiment no. 1
% then cur the iceshelf and add real sea ice forcings
% need Shapiro filter function shapiro1.m to smoothen iceshelf topography

%% dx dy files
% CENTERED Dimensions of grid, high resolution region
nx=200;
ny=400;
nz=50;

delta_z = 10;

dlat = 0.01;
dlon = 0.04;

acc = 'real*8';
ieee = 'b';

%% Grid file
dxfile = dlon*ones(1,nx);
dyfile = dlat*ones(1,ny);
dzfile = delta_z*ones(1,nz);


% The above are the ones with uniform high resolution
% Now we add the one with nonuniform linearly growing grid
% On both west and east sides of x, and north of y
ny_north_linear_ramp = 80;
ny_linear_ramp_added = ny_north_linear_ramp + ny; % 400 + 80

nx_west = 25;
nx_east = 25;
nx_linear_ramp_added = nx+nx_west+nx_east; % 200 + 25 + 25

dlon_max = 0.1;
dlat_max = 0.04;

west_add = linspace(dlon_max,dlon,nx_west);
east_add = linspace(dlon,dlon_max,nx_east);
north_add = linspace(dlat, dlat_max, ny_north_linear_ramp);

dxfile = [west_add dxfile east_add];
dyfile = [dyfile north_add];

%% Add the west and east lengthen layers, keep the same north dimension
% symmetric, making the total nx to be 360
% 200 + 25*2 + 55*2
nx_west_lengthen = 55;
nx_east_lengthen = 55;

west_add_lengthen = linspace(dlon_max,dlon_max,nx_west_lengthen);
east_add_lengthen = linspace(dlon_max,dlon_max,nx_east_lengthen);
dxfile = [west_add_lengthen dxfile east_add_lengthen];


% print out the bin files
fid=fopen('dxfile.bin','w','b'); fwrite(fid,dxfile,acc);fclose(fid);
fid=fopen('dyfile.bin','w','b'); fwrite(fid,dyfile,acc);fclose(fid);
fid=fopen('dzfile.bin','w','b'); fwrite(fid,dzfile,acc);fclose(fid);


%% longtitude and latitude vectors at face g and center c
northen_end_lat = -77; % starting at -77 lat

latg = northen_end_lat+cumsum(dyfile);
latc = latg+dyfile/2;

long = cumsum(dxfile); % changed from -80 to -77
lonc = long+dxfile/2;

%% Bathymetry parameters

% construct the center high resolution part first
% Point out the land part, bathy=0
% ice shelf part is about 50 points in x, 100 points in y

% H- Nominal depth of model (meters)
H = -500;
bathy_high_reso = ones(nx,ny)*H;
west_land_index = 76;
east_land_index = 127;

west_iceshelf_index = west_land_index+1;
east_iceshelf_index = east_land_index-1;

bathy_high_reso(:,1) = 0; % Wall on the south side

iceshelf_north_end_index = 100; % important, indicate land end
%
bathy_high_reso(1:(west_land_index), 1:iceshelf_north_end_index) = 0;
bathy_high_reso((east_land_index):nx, 1:iceshelf_north_end_index) = 0;
%

%% Round up the land corners

% Point land northwest
land_radius = 20;
i_center = west_land_index-land_radius;
j_center = iceshelf_north_end_index-land_radius;
for i = i_center:(west_land_index)
    for j = j_center:iceshelf_north_end_index
        if ((i-i_center)^2+(j-j_center)^2) > land_radius^2
            bathy_high_reso(i,j) = H;
        end
    end
end

% Point land northeast
i_center = east_land_index+land_radius;
j_center = iceshelf_north_end_index-land_radius;
for i = (east_land_index):i_center
    for j = j_center:iceshelf_north_end_index
        if ((i-i_center)^2+(j-j_center)^2) > land_radius^2
            bathy_high_reso(i,j) = H;
        end
    end
end


% Point iceshelf southwest
i_center = west_iceshelf_index+land_radius;
j_center = 2+land_radius;
for i = west_iceshelf_index:i_center
    for j = 2:j_center
        if ((i-i_center)^2+(j-j_center)^2) > land_radius^2
            bathy_high_reso(i,j) = 0;
        end
    end
end


% Point iceshelf southeast
i_center = east_iceshelf_index-land_radius;
j_center = 2+land_radius;
for i = i_center:east_iceshelf_index
    for j = 2:j_center
        if ((i-i_center)^2+(j-j_center)^2) > land_radius^2
            bathy_high_reso(i,j) = 0;
        end
    end
end

%% Bathymetry: Add the parts with low resolution
west_part = H*ones(nx_west,ny_linear_ramp_added);
east_part = H*ones(nx_east,ny_linear_ramp_added);
north_part_middle = H*ones(nx,ny_north_linear_ramp);
% For the north - only add the middle parts without the above west and east

bathy_combined = H*ones(nx_linear_ramp_added,ny_linear_ramp_added);

bathy_combined(1:nx_west, 1:ny_linear_ramp_added) = west_part;
bathy_combined(nx_west+1:nx_west+nx, 1:ny) = bathy_high_reso;
bathy_combined(nx_west+nx+1:nx_linear_ramp_added, 1:ny_linear_ramp_added) = east_part;
bathy_combined(nx_west+1:nx_west+nx, ny+1:ny_linear_ramp_added) = north_part_middle;

% modify and add new land with bathy_combined=0
bathy_combined(1:nx_west, 1:iceshelf_north_end_index) = 0;
bathy_combined(nx_west+nx+1:nx_linear_ramp_added, 1:iceshelf_north_end_index) = 0;

%% Bathymetry: Add the parts in the west and east with lengthen layers
west_lengthen = H*ones(nx_west_lengthen,ny_linear_ramp_added);
east_lengthen = H*ones(nx_east_lengthen,ny_linear_ramp_added);
bathy_combined = [west_lengthen; bathy_combined; east_lengthen];

% modify and add new land with bathy_combined=0
bathy_combined(1:nx_west_lengthen, 1:iceshelf_north_end_index) = 0;
bathy_combined((nx_west+nx+nx_west_lengthen+nx_east+1):(nx_linear_ramp_added+nx_west_lengthen+nx_east_lengthen), 1:iceshelf_north_end_index) = 0;


% Add the wall on the south boundary to avoid periodic BC
% bathy_combined(1,:) = 0; % Wall on the west side
bathy_combined(:,1) = 0; % Wall on the south side

fid=fopen('bathy.box','w','b'); fwrite(fid,bathy_combined,acc);fclose(fid);


%% Modified Ice shelf according to ISOMIP exp2

% A more smooth ice line with Shapiro filter
% -77 to -76 latitude, with -400m to -200m, then reduce to 0m at -76 latitude
% Then smooth the pointy part for the intersaction of two slopes

Hmin = -400; % deepest point of cavern
Hmax = -200; % shallowest point of cavern

pointy_latc_in = iceshelf_north_end_index-9;
dHdx = (Hmax-Hmin)/(-latg(1)-(-latc(pointy_latc_in)));

ice_line = (Hmin + dHdx*(latc-latg(1)));
ice_line(pointy_latc_in:(iceshelf_north_end_index+1)) = (ice_line(pointy_latc_in)-0)/(latc(pointy_latc_in)-...
    latc((iceshelf_north_end_index+1)))*(latc(pointy_latc_in:(iceshelf_north_end_index+1))-latc(pointy_latc_in))+ice_line(pointy_latc_in);


% Smoothen iceshelf topo for 50 times
smoothen_times = 50;

for i = 1:smoothen_times
    ice_line(1:(iceshelf_north_end_index-3)) = shapiro1(ice_line(1:(iceshelf_north_end_index-3)), 2, 1);
end

ice_line(:,(iceshelf_north_end_index+1):end) = 0; % ocean part, no iceshelf

% Plot iceshelf shape to check
figure
plot(latc, ice_line);
xlabel('latitude','fontsize',16)
ylabel('depth (m)','fontsize',16)
title('iceshelf shape','fontsize',18)
axis tight

icetopo = ones(size(bathy_combined,1),1)*ice_line;

% eliminate the part over land
icetopo(bathy_combined==0) = 0;

fid=fopen('icetopo.exp1','w','b'); fwrite(fid,icetopo,acc);fclose(fid);



%% wind and air external forcings

vwind1 = zeros(size(bathy_combined));

wind_main = 20;
wind_rest = 0;

% 50 grid points in x for the main field
xwind1 = 157; xwind2 = 206;
% 100 grid points in y for the main field
ywind1 = (iceshelf_north_end_index+1); ywind2 = ywind1+100;

vwind1(xwind1:xwind2, ywind1:ywind2) = wind_main;

wind_decay_length = 50;
wind_decay_frac = 1-(1:wind_decay_length)/wind_decay_length;

for k = 1:length(wind_decay_frac)
    vwind1(xwind1-k:xwind2+k, ywind2+k) = wind_decay_frac(k)*(wind_main-wind_rest)+wind_rest;
    vwind1(xwind1-k, ywind1:ywind2+k) = wind_decay_frac(k)*(wind_main-wind_rest)+wind_rest;
    vwind1(xwind2+k, ywind1:ywind2+k) = wind_decay_frac(k)*(wind_main-wind_rest)+wind_rest;
end


% relaxation points 3 cells each boundary for winds
% not really used here
vwind1(1:3,:) = 0;
vwind1((end-2):end,:) = 0;
vwind1(:,(end-2):end) = 0;

%% 20200322 overwrite- change the V-wind to a half-oval shape
mid_xwind12 = (xwind1+xwind2)/2;
vwind1 = zeros(size(bathy_combined));

wind_decay_frac_oval_flip = flip((1:wind_decay_length)/wind_decay_length);
for ki = length(wind_decay_frac_oval_flip):(-1):1
    for xi = 1:size(vwind1,1)
        for yi = ywind1:size(vwind1,2)

            if (xi-mid_xwind12)^2/(mid_xwind12-xwind1+ki)^2 + (yi-ywind1)^2/(ywind2-ywind1+ki)^2 <= 1
                vwind1(xi,yi) = wind_main*wind_decay_frac_oval_flip(ki);
            end
        end
    end
end

%% Switch between fluctuating wind and constant wind
wind_is_fluc = 1;
wind_is_const = (~wind_is_fluc);



% fluctuating wind

% 12-hour period, 200 days



load wind_scale_6hr_200days_sporadic_data.mat

if wind_is_fluc == 1

    for j = 1:801
        if j == 1
            vwind(:,:,j) = zeros(size(vwind1)); % first period is time
        else
            vwind(:,:,j) = vwind1*2 *wind_value_scale_6hr(j) /40*40; % fluctuating to 50 m/s
        end
    end

end



fid=fopen('vwind.bin','w','b'); fwrite(fid,vwind,acc);fclose(fid);


add_Uwind = 1;

if add_Uwind == 1
    % 20200116- add uwind=-10m/s, constant over the time
    uwind_value = -10;
    uwind1 = uwind_value*ones(size(bathy_combined));


    % 20210628 - add ramp of Uwind near all the north/east/west boundaries
    add_uwind_BC_ramp_flag = 1;
    if add_uwind_BC_ramp_flag == 1
        uwind1 = function_boundary_ramp_inwards_2d_setup(bathy_combined,0,uwind_value,0,30);
        uwind1(30:(end-30+1),1:iceshelf_north_end_index) = uwind_value;
    end



    % remember to confirm the period length in data.exf
    % and also confirm data.obcs setup with uice and vice BC

    % default: const Uwind
    for j = 1:801
        if j == 1
            uwind(:,:,j) = zeros(size(uwind1)); % first period is time
        else
            uwind(:,:,j) = uwind1 * 1; % constant to 10 m/s
        end
    end

    fid=fopen('uwind.bin','w','b'); fwrite(fid,uwind,acc);fclose(fid);

end


%% Airtemp- const everywhere

airtemp_rest = 0;
airtemp_main = -20;
airtemp_const_rest_length = 0;
airtemp_ramp_to_center_length = 0;
airtemp1 = function_boundary_ramp_inwards_2d_setup(bathy_combined,airtemp_rest,airtemp_main,airtemp_const_rest_length,airtemp_ramp_to_center_length);

airtemp2 = airtemp1;
airtemp(:,:,1) = airtemp1;
airtemp(:,:,2) = airtemp2;

fid=fopen('airtempfield.bin','w','b'); fwrite(fid,airtemp,acc);fclose(fid);

%% Humidity field- const everywhere
humidfield_value = 0.0005;
humidfield1 = humidfield_value*ones(size(bathy_combined));
humidfield2 = humidfield1;

humidfield(:,:,1) = humidfield1;
humidfield(:,:,2) = humidfield2;

fid=fopen('humidfield.bin','w','b'); fwrite(fid,humidfield,acc);fclose(fid);

%% Longwave radiation field- const everywhere
lwdownfield_main = 200;
lwdownfield_rest = 270;

lwdownfield1 = function_boundary_ramp_inwards_2d_setup(bathy_combined,lwdownfield_rest,lwdownfield_main,airtemp_const_rest_length,airtemp_ramp_to_center_length);


lwdownfield2 = lwdownfield1;
lwdownfield(:,:,1) = lwdownfield1;
lwdownfield(:,:,2) = lwdownfield2;

fid=fopen('lwdownfield.bin','w','b'); fwrite(fid,lwdownfield,acc);fclose(fid);

%% Sea ice initial files
SIarea_init_value = 1;
SIarea_init = SIarea_init_value*ones(size(bathy_combined));
SIarea_init(bathy_combined == 0) = 0; % get rid of the land
SIarea_init(icetopo ~= 0) = 0; % get rid of ice shelf, ice tongue and pack ice
fid=fopen('SIarea_init.bin','w','b'); fwrite(fid,SIarea_init,acc);fclose(fid);

SIheff_init_value = 0.5;
SIheff_init = SIheff_init_value*ones(size(bathy_combined));
SIheff_init(bathy_combined == 0) = 0; % get rid of the land
SIheff_init(icetopo ~= 0) = 0; % get rid of ice shelf, ice tongue and pack ice
fid=fopen('SIheff_init.bin','w','b'); fwrite(fid,SIheff_init,acc);fclose(fid);

%% Empty external forcing- use 0 eyerywhere
empty2dfield1 = 0.0*ones(size(bathy_combined));
empty2dfield2 = empty2dfield1;

empty2dfield(:,:,1) = empty2dfield1;
empty2dfield(:,:,2) = empty2dfield2;

fid=fopen('empty2dfield.bin','w','b'); fwrite(fid,empty2dfield,acc);fclose(fid);



%% Seaice OBCS Nx/y * time levels
% ice area = 1 west east, with decay northward
% 20 points used for decaying, about 0.8deg in lat
% north point is 0
ice_area = 0;
ice_area_bc_decay_num = 20;
ice_area_bc_decay = linspace(ice_area,0,ice_area_bc_decay_num);
ice_area_bc_decay = ice_area_bc_decay';

iceOBCSawest1 = ice_area*ones(size(bathy_combined,2),1);
iceOBCSawest1((end-ice_area_bc_decay_num+1):end)=ice_area_bc_decay;
iceOBCSawest1(1:iceshelf_north_end_index,:) = 0; % eliminate the land part
iceOBCSawest2 = iceOBCSawest1;
iceOBCSawest(:,1) = iceOBCSawest1*0.0;
iceOBCSawest(:,2) = iceOBCSawest2*0.0;
fid=fopen('iceOBCSawest.bin','w','b'); fwrite(fid,iceOBCSawest,acc);fclose(fid);


iceOBCSaeast = iceOBCSawest;
fid=fopen('iceOBCSaeast.bin','w','b'); fwrite(fid,iceOBCSaeast,acc);fclose(fid);

iceOBCSanorth1 = 0*ones(size(bathy_combined,1),1);
iceOBCSanorth2 = iceOBCSanorth1;
iceOBCSanorth(:,1) = iceOBCSanorth1;
iceOBCSanorth(:,2) = iceOBCSanorth2;
fid=fopen('iceOBCSanorth.bin','w','b'); fwrite(fid,iceOBCSanorth,acc);fclose(fid);


% ice thickness
% enforcing the same pattern as ice area bc

iceOBCShwest = 0.0*iceOBCSawest;
fid=fopen('iceOBCShwest.bin','w','b'); fwrite(fid,iceOBCShwest,acc);fclose(fid);

iceOBCSheast = 0.0*iceOBCSaeast;
fid=fopen('iceOBCSheast.bin','w','b'); fwrite(fid,iceOBCSheast,acc);fclose(fid);

iceOBCShnorth = 0.0*iceOBCSanorth;
fid=fopen('iceOBCShnorth.bin','w','b'); fwrite(fid,iceOBCShnorth,acc);fclose(fid);



%% add on 01/22- add sea ice u-velocity on the east/west boundaries

uiceOBCSwest = 0.0*ones(size(iceOBCSawest));
uiceOBCSwest(1:iceshelf_north_end_index,:) = 0;
fid=fopen('uiceOBCSwest.bin','w','b'); fwrite(fid,uiceOBCSwest,acc);fclose(fid);

uiceOBCSeast = uiceOBCSwest;


% enforce east BC Uice=0
uiceOBCSeast = 0.0*uiceOBCSeast;
fid=fopen('uiceOBCSeast.bin','w','b'); fwrite(fid,uiceOBCSeast,acc);fclose(fid);



%% Add the parts for ptracers surface intial file

ptracers2d=ones(size(bathy_combined));
ptracers2d(:,1:iceshelf_north_end_index) = 0;

ptracers3d = zeros(size(bathy_combined,1),size(bathy_combined,2),nz);



fid=fopen('ptracers_file.bin','w',ieee); fwrite(fid,ptracers3d,acc); fclose(fid);


%% Add mask file for Ptracers, use the below for RBCS

% The west/east/north boundary cells relaxation for ptracers
ptracers_mask = zeros(size(ptracers3d));

ptr_rbcs_ramp_length = 10;

ptracers_boundary_value = 1/24; % 24*3600s 1-daily relaxation

ptracers2d_bc = function_boundary_ramp_inwards_2d_setup(bathy_combined,ptracers_boundary_value,0,0,ptr_rbcs_ramp_length);

for depth_in = 1:nz
    ptracers_mask(:,:,depth_in) = ptracers2d_bc;
end

% The surface relaxation for constant supply of ptracers (to mimic salt)
ptracers_mask(ptracers3d~=0) = 1;

fid=fopen('rbcs_mask_ptracers.bin','w',ieee); fwrite(fid,ptracers_mask,acc); fclose(fid);
